%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Cambridge University Press 2010
% Book: Karl Mathia (2010), Robotics for Electronics Manufacturing - 
% Principles and Applications in Cleanroom Manufacturing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Chapter 5, Example 5.1:
% Kinematic structure of a 3-link planar robot.
% Refer to the description in the book for further details.
%
% The following code can be executed with Matlab(R)* or GNU Octave. The
% Octave plot function uses gnuplot and may cause the plot window to freeze
% in rare cases.
% *Matlab(R) is a registered trademark of The MathWorks.

clear
clc
    
% Constant robot parameters
origin = [0;0];
L1 = 1;         % link length: link 1
L2 = 1;         % link length: link 2
L3 = 1;         % link length: link 2
Q1MIN = 0;      % joint space limit: q1 minimum
Q1MAX = pi/2;   % joint space limit: q1 maximum
Q2MIN = 0;      % joint space limit: q2 minimum
Q2MAX = pi/2;   % joint space limit: q2 maximum
Q3MIN = 0;      % joint space limit: q2 minimum
Q3MAX = pi/2;   % joint space limit: q2 maximum
% Points A, B, C on joint space boundary
A = [Q1MAX/2 ; 0 ; 0 ];
B = [Q1MAX ; Q2MAX ; Q3MAX/2 ];
C = [Q1MAX ; Q2MAX/2 ; 0];


% Illustrate work space boundaries by sweeping through joint space. Plot
% the end-effector origin for all selected joint displacements (angles).

% Work space boundaries: sweep through q1 (fully extended arm)
q2 = Q2MIN;
q3 = Q3MIN;
k = 1;
for q1 = Q1MIN:pi/18:Q1MAX
    boundary11(:,k) = fkinematics_r3p1(L1,L2,L3,q1,q2,q3);
    k = k+1;    
end
q2 = Q2MAX;
q3 = Q3MAX;
k = 1;
for q1 = Q1MIN:pi/18:Q1MAX
    boundary12(:,k) = fkinematics_r3p1(L1,L2,L3,q1,q2,q3);
    k = k+1;    
end

% Work space boundaries: sweep through q2
q1 = Q1MAX;
q3 = Q3MIN;
k = 1;
for q2 = Q2MIN:pi/18:Q2MAX
    boundary21(:,k) = fkinematics_r3p1(L1,L2,L3,q1,q2,q3);
    k = k+1;    
end
q1 = Q1MIN;
q3 = Q3MAX;
k = 1;
for q2 = Q2MIN:pi/18:Q2MAX
    boundary22(:,k) = fkinematics_r3p1(L1,L2,L3,q1,q2,q3);
    k = k+1;    
end

% Work space boundaries: sweep through q3
q1 = Q1MAX;
q2 = Q2MAX;
k = 1;
for q3 = Q3MIN:pi/18:Q3MAX
    boundary31(:,k) = fkinematics_r3p1(L1,L2,L3,q1,q2,q3);
    k = k+1;
end
q1 = Q1MIN;
q2 = Q2MIN;
k = 1;
for q3 = Q3MIN:pi/18:Q3MAX
    boundary32(:,k) = fkinematics_r3p1(L1,L2,L3,q1,q2,q3);
    k = k+1;
end


% Display results
disp(' ')
disp('******************************************************************')
disp('***         Copyright Cambridge University Press 2010          ***')
disp('*** Karl Mathia (2010). Robotics for Electronics Manufacturing ***')
disp('***                  Chapter 5, Example 5.1                    ***')
disp('******************************************************************')
disp(' ')
disp('Kinematic structure of a 3-link planar robot.')
disp('Please refer to the description in the book for further details.')
disp(' ')
disp('Plotting the workspace boundaries and the three arm configurations')
disp('at points A, B, and C...')
disp(' ')
disp('******************************************************************')

% Plot work space boundary
plot(boundary11(1,:),boundary11(2,:),'--k') % link 1 moves
hold on
plot(boundary12(1,:),boundary12(2,:),'--k') % link 1 moves
plot(boundary21(1,:),boundary21(2,:),':k')  % link 2 moves
plot(boundary22(1,:),boundary22(2,:),':k')  % link 2 moves
plot(boundary31(1,:),boundary31(2,:),'-.k') % link 3 moves
plot(boundary32(1,:),boundary32(2,:),'-.k') % link 3 moves

% Add three robot arm configurations to the plot
% Compute and plot the manipulator configurations at points A, B, C.
configs = [A,B,C];

for k = 1:size(configs,2)
    [ee,l1,l2,l3] = fkinematics_r3p1(L1,L2,L2,configs(1,k),configs(2,k),configs(3,k));
    
    % plot first (inner) link l1
    plot([origin(1),l1(1)],[origin(2),l1(2)],'-k','LineWidth',2)
    plot(l1(1),l1(2),'ok','LineWidth',2)
    plot(origin(1),origin(2),'ok','LineWidth',2)
    
    % plot second (outer) link l2, end-point is end-effector origin ee
    l12 = l1 + l2;
    plot([l1(1),l12(1)],[l1(2),l12(2)],'-k','LineWidth',2)
    plot(l1(1),l1(2),'ok','LineWidth',2)
    plot(l12(1),l12(2),'ok','LineWidth',2)
    
    % plot end-effector (link l3), end-point is end-effector tool ee
    l13 = l1 + l2 + l3;
    plot([l12(1),l13(1)],[l12(2),l13(2)],'-k','LineWidth',2)
    plot(l12(1),l12(2),'ok','LineWidth',2)
    plot(l13(1),l13(2),'ok','LineWidth',2)

end

hold off
axis equal
axis([-2.5 +3.5 -0.5 +3.5])
xlabel('x_0')
ylabel('y_0')

%%%