%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Cambridge University Press 2010
% Book: Karl Mathia (2010), Robotics for Electronics Manufacturing - 
% Principles and Applications in Cleanroom Manufacturing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Chapter 7, Example 7.3:
% Computing the 2D repeatability and accuracy.
% Refer to the description in the book for further details.
%
% The following code can be executed with Matlab(R)* or GNU Octave.
% *Matlab(R) is a registered trademark of The MathWorks.

clear
clc
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Generate test data                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Load the test data measured by the two cameras of the measurement system,
% the vision-based repeatability fixture. The measurement resolution is 
% 1 micrometer [um].

% Camera measurements:
load camera1.dat
load camera2.dat
xc1 = camera1;
xc2 = camera2;
clear camera1
clear camera2

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             Data analysis                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Commanded wafer center position is at the origin of the measurement 
% system's coordinate frame 
xcmd = [0 ; 0];             % commanded end-effector position in [um]
D = 180000;                 % distance between patterns [um]
qcmd = 0;                   % commanded wafer angle [deg]

% Wafer center position errors
xw = 1/2 * (xc1 + xc2);

% Wafer angle errors
tmp = (xc1(2,:)-xc2(2,:))/D;
qw = atan(tmp)*180/pi;       % wafer angle in [deg]

% Positioning accuracy 
N = length(xw);

% Positioning accuracy is obtained with the mean wafer position
xw_avg = 1/N * [sum(xw(1,:)) ; sum(xw(2,:))];
ap = norm(xw_avg - xcmd);

% The positioning repeatability is
for i = 1:N
    d(i) = norm(xw(:,i) - xw_avg);
end
davg = 1/N * sum(d);
sd = std(d);
rp = davg + 3 * sd;

% Orientation accuracy is obtained with the mean wafer angle
qavg = 1/N * sum(qw);
ao = norm(qavg - qcmd);

% The orientation repeatability
for i = 1:N
    q(i) = norm(qw(:,i) - qavg);
end
qavg = 1/N * sum(q);
sq = std(q);
ro = qavg + 3 * sq;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                         Display results                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(' ')
disp('******************************************************************')
disp('***         Copyright Cambridge University Press 2010          ***')
disp('*** Karl Mathia (2010). Robotics for Electronics Manufacturing ***')
disp('***                  Chapter 7, Example 7.3                    ***')
disp('******************************************************************')
disp(' ')
disp('Computing the 2D repeatability and accuracy.')
disp('Please refer to the description in the book for further details.')
disp(' ')
disp('Camera 1 measurements [um]:')
disp(xc1)
disp('Camera 2 measurements [um]:')
disp(xc2)
disp('Wafer center position errors [um]:')
disp(xw)
disp('Average wafer center position error [um]:')
disp(xw_avg)
disp('Wafer angle errors [deg]:')
disp(qw)
disp('Positioning accuracy ap [um]:')
disp(ap)
disp('Positioning repeatability rp [um]:')
disp(rp)
disp('Orientation accuracy ao [deg]:')
disp(ao)
disp('Orientation repeatability ro [deg]:')
disp(ro)
disp('******************************************************************')

%%%
