%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Cambridge University Press 2010
% Book: Karl Mathia (2010), Robotics for Electronics Manufacturing - 
% Principles and Applications in Cleanroom Manufacturing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Chapter 7, Example 7.5:
% Computing the 3D robot repeatability and accuracy from processed 3D test 
% data (laser-based) per ISO standard 9283.
% Refer to the description in the book for further details.
%
% The following code can be executed with Matlab(R)* or GNU Octave. The
% Octave plot function uses gnuplot and may cause the plot window to freeze
% in rare cases.
% *Matlab(R) is a registered trademark of The MathWorks.

clear
clc
    
% Set commanded robot pose Xc = (xc,yx,zc), i.e. teh commanded end-effector
% position to zero. This reflects a coordinate transform by the data's mean
% value.  
xc = 0;
yc = 0;
zc = 0;

% Load 3D sampled robot pose data (repeatability test data)
load Laser_repeatability_3D.txt
Xa = Laser_repeatability_3D;   % N 'attained' robot poses Xa = (xa,ya,za)
N = length(Xa);
xa = Xa(:,2);
ya = Xa(:,3);
za = Xa(:,4);

% Mean robot pose Xm = [xm;ym;zm]
xm = mean(xa);
ym = mean(ya);
zm = mean(za);
Xm = [xm;ym;zm];

%%%%% Positioning accuracy ap per ISO 9283, using the Euclidean norm
ap = sqrt((xm - xc)^2 + (ym - yc)^2 + (zm - zc)^2);


%%%% Positioning repeatability per ISO Standard 9283 %%%%

% Standard deviation for all distances d(i) between attained poses
% xa(i) and the mean pose xm

% First step: all distances d(i) between attained poses xa(i) and the mean
% pose xm: Euclidean norm is used as distance. 
% Note: the Matlab function d(i) = norm([Xa(i,2:3),0]'-Xm) could be used.
for i = 1:N
    d(i) = sqrt((xa(i)-xm)^2 + (ya(i)-ym)^2 + (za(i)-zm)^2);
    %dtmp(i) = norm([Xa(i,2:3),0]'-Xm);     % for verification only
end

% Second step: average (mean) distance dm of all distances d(i)
% Note: the Matlab function dm = mean(d) could be used.
dsum = 0;   % sum of all Euclidean norms, initiatization
for i = 1:N
    dsum = dsum + d(i);
end
dm = dsum/N;
%dm = mean(d);     % for verification only

% Third step: standard deviation dstd for all d(i)
% Note: the Matlab function dstd = std(d) could be used.
dsum2 = 0;      % initialization
for i = 1:N
    dsum2 = dsum2 + (d(i)-dm)^2;
end
% std deviation
dstd = sqrt(dsum2/(N-1));
%dstd = std(d);     % for verification only

%%%% Positioning repeatability rp per ISO Standard 9283 %%%%
rp = dm + 3*dstd;


% Count the number of data outside of repeatability circle.  In theory the
% 3 sigma limit includes 99.73% of all samples.
nout = 0; 
for i = 1:N
    if d(i)>rp
        nout = nout + 1;
    end
end
noutp = nout/N*100;


% Display results
disp(' ')
disp('******************************************************************')
disp('***         Copyright Cambridge University Press 2010          ***')
disp('*** Karl Mathia (2010). Robotics for Electronics Manufacturing ***')
disp('***                  Chapter 7, Example 7.5                    ***')
disp('******************************************************************')
disp(' ')
disp('Computing the 3D robot repeatability and accuracy from processed')
disp('test data (laser-based) per ISO standard 9283.')
disp('Please refer to the description in the book for further details.')
disp(' ')
disp('Repeatability Test Results per ISO Standard 9283:')
disp(['  Test samples       :      N  = ',num2str(N)])
disp(['  Commanded pose [mm]:      Xc = (',num2str(xc),',',num2str(yc),',',num2str(zc),')'])
disp(['  Mean pose [mm]     :      Xm = (',num2str(xm),',',num2str(ym),',',num2str(zm),')'])
disp(['  Accuracy [mm]      :      ap = ',num2str(ap)])
disp(['  Mean distance [mm] :      dm = ',num2str(dm)])
disp(['  Std. deviation [mm]:    dstd = ',num2str(dstd)])
disp(['  Repeatability [mm] :      rp = ',num2str(rp)])
disp(['  Outliers           :    nout = ',num2str(nout)])
disp(['  Outliers [%]       :    nout = ',num2str(noutp)])
disp(' ')
disp('Plotting the results...')
disp(' ')
disp('Change the view of Figure 2 using "view(azimuth,elevation)":')
disp(' x/y-view:  view(    0,90), azimuth =  90,   elevation = 90')
disp(' x/z-view:  view(    0, 0), azimuth =   0,   elevation =  0')
disp(' y/z-view:  view(    90,0), azimuth =  90,   elevation = 90')
disp(' 3D-view :  view(-37.5,30), azimuth = -37.5, elevation = 30')
disp('All values are in degrees.')
disp(' ')
disp('******************************************************************')


% 3D Cartesian plots of commanded pose, mean pose, and all attained poses
xc = 1000*xc;       % convert [mm] to [um]
yc = 1000*yc;       % convert [mm] to [um]
zc = 1000*zc;       % convert [mm] to [um]
xm = 1000*xm;       % convert [mm] to [um]
ym = 1000*ym;       % convert [mm] to [um]
zm = 1000*zm;       % convert [mm] to [um]
xa = 1000*xa;       % convert [mm] to [um]
ya = 1000*ya;       % convert [mm] to [um]
za = 1000*za;       % convert [mm] to [um]
rp = 1000*rp;       % convert [mm] to [um]

figure(1)
plot3(xc,yc,zc,'xb','LineWidth',2,'MarkerSize',10)
hold on
plot3(xm,ym,zm,'or','LineWidth',2,'MarkerSize',10)
plot3(xa,ya,za,'.k')
[Xplot,Yplot,Zplot] = sphere(40);
Xplot = Xplot*rp + xm;
Yplot = Yplot*rp + ym;
Zplot = Zplot*rp + zm;
mesh(Xplot,Yplot,Zplot)
axis([(xm-1.1*rp) (xm+1.1*rp) (ym-1.1*rp) (ym+1.1*rp) (zm-1.1*rp) (zm+1.1*rp)])
axis square
grid on
hold off
xlabel('x [mm]')
ylabel('y [mm]')
zlabel('z [mm]')

figure(2)
plot3(xc,yc,zc,'xk','LineWidth',2,'MarkerSize',10)
hold on
plot3(xm,ym,zm,'ok','LineWidth',2,'MarkerSize',10)
plot3(xa,ya,za,'.k')
% add repeatability circle about mean pose to plot
acir = 0:0.1:2.01*pi;  % angles for circle
co = rp * cos(acir);
si = rp * sin(acir);
cir1 = ones(size(co));
hold on
plot3(xm+co,ym+si,zm*cir1,'--k')
plot3(xm+co,ym*cir1,zm+si,'--k')
plot3(xm*cir1,ym+co,zm+si,'--k')
hold off
axis([(xm-1.1*rp) (xm+1.1*rp) (ym-1.1*rp) (ym+1.1*rp) (zm-1.1*rp) (zm+1.1*rp)])
axis square
grid on
view(0,90)   % view([x,y,z]) sets view point
xlabel('x [um]')
ylabel('y [um]')
zlabel('z [um]')

%%%