%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Cambridge University Press 2010
% Book: Karl Mathia (2010), Robotics for Electronics Manufacturing - 
% Principles and Applications in Cleanroom Manufacturing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Chapter 7, Example 7.6:
% Computing the path positioning accuracy and repeatability of a straight 
% 3D path from hypothetical 3D test data per ISO standard 9283.
% Refer to the description in the book for further details.
%
% The following code can be executed with Matlab(R)* or GNU Octave. The
% Octave plot function uses gnuplot and may cause the plot window to freeze
% in rare cases.
% *Matlab(R) is a registered trademark of The MathWorks.

clear
clc
    
% Simple, hypothetical data (manually created):
% Commanded path xc(i), i=1,..,M: x-axis, so all y/z points are (0,0).
xc = [	0.2	0.4	0.6	0.8	1.0	1.2
        0.0	0.0	0.0	0.0	0.0	0.0
        0.0	0.0	0.0	0.0	0.0	0.0];

% N = 3 attained paths are measured at x-positions x(i), i=1,..M, with M=6.
% These hypothetical data are normally distributed and were created with
% Matlab's randn() function.
N = 3;
x1 = [
    0.1930    0.3709    0.5255    0.8264    0.9654    1.2450
   -0.0565   -0.0448    0.0157    0.0172    0.0340   -0.1062
   -0.0146    0.0124   -0.1013    0.0379   -0.0536    0.0142];
x2 = [
    0.1633    0.3832    0.6507    0.8199    0.9856    1.2166
   -0.0387    0.0485   -0.0238    0.0558   -0.0686   -0.0499
    0.0076   -0.0054    0.0034    0.0310   -0.0343    0.0146];
x3 = [
    0.2554    0.4203    0.5487    0.7868    0.9575    1.2380
    0.0122    0.0608    0.0103    0.1248    0.0406   -0.0856
    0.0082    0.0724    0.0294    0.0428    0.0350    0.0769];


%%%% Path (positioning) accuracy per ISO Standard 9283 %%%%
M = size(xc,2);          % number of measurements M

% Mean path xm(i), i=1,..,M, for N attained paths
xm = (x1 + x2 + x3)/N;

% Path (positioning) accuracies along attained path xj(i), using the 
% Euclidean norm:
for i = 1:M
    % Mean position errors em(i), i=1,..,M, between mean path xm(i) and
    % comanded path position xc(i)
    em = xm(:,i) - xc(:,i);
    
    % Norm (length) of all M mean position errors along path
    emd(i) = sqrt(em(1)^2 + em(2)^2 + em(3)^2);
end

% Path (positioning) accuracy is the maximum of all mean position errors
at = max(emd);


%%%% Path (positioning) repeatability per ISO Standard 9283 %%%%

% First step: compute all distances (Euclidean norm) d(i) between attained 
% paths xj(i), j=1,..,N and the mean path xm(i). % Difference xdj(i)
% between attained path xj(i) and mean path xm(i), i=1,..,M. Note: the 
% Matlab function norm(.) is not used here for clarity.

xd1 = x1 - xm;
xd2 = x2 - xm;
xd3 = x3 - xm;

% Distance (Euclidean norm) between path position difference xdj(i) and the
% mean path xm(i) at the i-th measured position 
for i = 1:M
    d1(i) = sqrt(xd1(1,i)^2 + xd1(2,i)^2 + xd1(3,i)^2);
    d2(i) = sqrt(xd2(1,i)^2 + xd2(2,i)^2 + xd2(3,i)^2);
    d3(i) = sqrt(xd3(1,i)^2 + xd3(2,i)^2 + xd3(3,i)^2);
    %d1_tmp(i) = norm(xd1(:,i));        % for verification
    %d2_tmp(i) = norm(xd2(:,i));        % for verification
    %d3_tmp(i) = norm(xd3(:,i));        % for verification
end

% Second step: % Mean (average) distance dmj(i) along attained path xj(i). 
% Note: the Matlab function norm(.) is not used here for clarity.
dm = (d1 + d2 + d3)/N;
%dm_tmp = mean([d1 ; d2 ; d3]);         % for verification

% Third step: standard deviation sigma(i) for all attained paths xj(i),
% j=,1,..,N. Note: the Matlab function std(.) is not used here for clarity.
dd1 = d1 - dm;
dd2 = d2 - dm;
dd3 = d3 - dm;
for i = 1:M
    sigma(i) = sqrt((dd1(i)^2 + dd2(i)^2 + dd3(i)^2)/(N-1));
    %sigma_tmp(i) = std([dd1(i) ; dd2(i) ; dd3(i)]);    % for verification
end

% Fourth step: a) path (positioning) repeatability rtp(i) at the i-th position
% for N attained paths, and b) the total path rpeatability rT
rtp = dm + 3*sigma;
rt = max(rtp);


% Display results
disp(' ')
disp('******************************************************************')
disp('***         Copyright Cambridge University Press 2010          ***')
disp('*** Karl Mathia (2010). Robotics for Electronics Manufacturing ***')
disp('***                  Chapter 7, Example 7.6                    ***')
disp('******************************************************************')
disp(' ')
disp('Computing the path positioning accuracy and repeatability of a') 
disp('straight 3D path from hypothetical 3D test data per ISO 9283.')
disp('Please refer to the description in the book for further details.')
disp(' ')
disp('3D path positioning results:')
disp(['  Attained paths, j=,..,N:    N  = ',num2str(N)])
disp(['  Samples per path, i=1,..,M: M  = ',num2str(M)])
disp(['  Commanded path:']);        disp('  xc(i) = ');    disp(xc)
disp(['  Attained path 1 (j=1):']); disp('  x1(i) = ');    disp(x1)
disp(['  Attained path 2 (j=2):']); disp('  x2(i) = ');    disp(x2)
disp(['  Attained path 3 (j=3):']); disp('  x3(i) = ');    disp(x3)
disp(['  Mean path:']);             disp('  xm(i) = ');    disp(xm)
disp(['  Mean position errors:']);  disp('  emd(i) = ');   disp(emd)
disp(['  Mean distances:']);        disp('  dm(i) = ');    disp(dm)
disp(['  Standard deviations:']);   disp('  sigma(i) = '); disp(sigma)
disp(['Results:'])
disp(['  Path accuracy:       at = ',num2str(at)])
disp(['  Path repeatability:  rt = ',num2str(rt)])
disp(' ')
disp('******************************************************************')

%%%
