%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Cambridge University Press 2010
% Book: Karl Mathia (2010), Robotics for Electronics Manufacturing - 
% Principles and Applications in Cleanroom Manufacturing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Chapter 7, Example 7.7:
% Computing the path positioning accuracy and repeatability of a straight 
% 3D path from real-world 3D (laser-based) test data per ISO standard 
% 9283.
% Refer to the description in the book for further details.
%
% The following code can be executed with Matlab(R)* or GNU Octave. The
% Octave plot function uses gnuplot and may cause the plot window to freeze
% in rare cases.
% *Matlab(R) is a registered trademark of The MathWorks.

clear
clc
    
% Real-world data from a 300mm wafer handling robot
load static_path_cmd_att.txt    % load static, commanded and attained paths
load dynamic_path_att.txt       % load dynamic, attained path

% Commanded path xc(i), i=1,..,M: x-axis, so all y/z points are (0,0).
xc = static_path_cmd_att(:,2:4);
xc = xc';
M = length(xc);          % number of measurements M

% N attained paths are measured at x-positions x(i), i=1,..M.
N = 3;
x1 = static_path_cmd_att(:,5:7);
x1 = x1';
x2 = static_path_cmd_att(:,8:10);
x2 = x2';
x3 = static_path_cmd_att(:,11:13);
x3 = x3';

% Dynamic, attained paths
xd = dynamic_path_att(:,2:4);
xd = xd';
Md = length(xd);          % number of measurements M


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%       3D path (positioning) accuracy and repeadability (x,y,z)         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 3D path (positioning) accuracy
% Mean path xm(i), i=1,..,M, for N attained paths
xm = (x1 + x2 + x3)/N;

% Path (positioning) accuracies along attained path xj(i), using the 
% Euclidean norm:
for i = 1:M
    % Mean position errors em(i), i=1,..,M, between mean path position
    % xm(i) and commanded path position xc(i)
    em(:,i) = xm(:,i) - xc(:,i);
    
    % Norm (length) of all M mean position errors along path
    emd(i) = sqrt(em(1,i)^2 + em(2,i)^2 + em(3,i)^2);
end

% Path (positioning) accuracy is the maximum of all mean position errors
[at_3D,imax_3D] = max(emd);


% Compute 3D path (positioning) repeatability

% First step: compute all distances (Euclidean norm) d(i) between attained 
% paths xj(i), j=1,..,N and the mean path xm(i). 
% Difference xdj(i) between attained path xj(i) and mean path xm(i), 
% i=1,..,M. Note: the Matlab function norm() is not used here for clarity.

xd1 = x1 - xm;
xd2 = x2 - xm;
xd3 = x3 - xm;

% Distance (Euclidean norm) between path position difference xdj(i) and the
% mean path xm(i) at the i-th measured position 
for i = 1:M
    d1(i) = sqrt(xd1(1,i)^2 + xd1(2,i)^2 + xd1(3,i)^2);
    d2(i) = sqrt(xd2(1,i)^2 + xd2(2,i)^2 + xd2(3,i)^2);
    d3(i) = sqrt(xd3(1,i)^2 + xd3(2,i)^2 + xd3(3,i)^2);
    %d1_tmp(i) = norm(xd1(:,i));        % for verification only
    %d2_tmp(i) = norm(xd2(:,i));        % for verification only
    %d3_tmp(i) = norm(xd3(:,i));        % for verification only
end

% Second step: mean (average) distance dmj(i) along attained path xj(i). 
% Note: the Matlab function norm() is not used here for clarity.
dm = (d1 + d2 + d3)/N;
%dm_tmp = mean([d1 ; d2 ; d3]);         % for verification only

% Third step: standard deviation sigma(i) for all attained paths xj(i),
% j=,1,..,N. Note: the Matlab function std() is not used here for clarity.
dd1 = d1 - dm;
dd2 = d2 - dm;
dd3 = d3 - dm;
for i = 1:M
    sigma(i) = sqrt((dd1(i)^2 + dd2(i)^2 + dd3(i)^2)/(N-1));
    %sigma_tmp(i) = std([dd1(i) ; dd2(i) ; dd3(i)]);    % for verification
end

% Fourth step: a) path (positioning) repeatability rtp(i) at the i-th position
% for N attained paths, and b) the total path rpeatability rT
rtp = dm + 3*sigma;
rt_3D = max(rtp);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     2D path (positioning) accuracy and repeatability (x=0,y,z)         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Calculate the path accuracy and repeatability in 2D (y,z) simply by
% replacing the actual x-coordinates of x1, x2, x3 with the commanded
% x-coordinate of xc. That will result in a zero x-error.
x1_2D = x1;
x1_2D(1,:) = xc(1,:);
x2_2D = x2;
x2_2D(1,:) = xc(1,:);
x3_2D = x3;
x3_2D(1,:) = xc(1,:);

% 2D path (positioning) accuracy
% Mean path xm(i), i=1,..,M, for N attained paths
xm = (x1_2D + x2_2D + x3_2D)/N;

% Path (positioning) accuracies along attained path xj(i), using the 
% Euclidean norm:
for i = 1:M
    % Mean position errors em(i), i=1,..,M, between mean path position
    % xm(i) and commanded path position xc(i)
    em(:,i) = xm(:,i) - xc(:,i);
    
    % Norm (length) of all M mean position errors along path
    emd(i) = sqrt(em(1,i)^2 + em(2,i)^2 + em(3,i)^2);
end

% Path (positioning) accuracy is the maximum of all mean position errors
[at_2D,imax_2D] = max(emd);


% 2D path (positioning) repeatability 

% First step: compute all distances (Euclidean norm) d(i) between attained 
% paths xj(i), j=1,..,N and the mean path xm(i). % Difference xdj(i)
% between attained path xj(i) and mean path xm(i), i=1,..,M. Note: the 
% Matlab function norm() is not used here for clarity.

xd1 = x1_2D - xm;
xd2 = x2_2D - xm;
xd3 = x3_2D - xm;

% Distance (Euclidean norm) between path position difference xdj(i) and the
% mean path xm(i) at the i-th measured position 
for i = 1:M
    d1(i) = sqrt(xd1(1,i)^2 + xd1(2,i)^2 + xd1(3,i)^2);
    d2(i) = sqrt(xd2(1,i)^2 + xd2(2,i)^2 + xd2(3,i)^2);
    d3(i) = sqrt(xd3(1,i)^2 + xd3(2,i)^2 + xd3(3,i)^2);
    %d1_tmp(i) = norm(xd1(:,i));        % for verification only
    %d2_tmp(i) = norm(xd2(:,i));        % for verification only
    %d3_tmp(i) = norm(xd3(:,i));        % for verification only
end

% Second step: % Mean (average) distance dmj(i) along attained path xj(i). 
% Note: the Matlab function norm(.) is not used here for clarity.
dm = (d1 + d2 + d3)/N;
%dm_tmp = mean([d1 ; d2 ; d3]);         % for verification only

% Third step: standard deviation sigma(i) for all attained paths xj(i),
% j=,1,..,N. Note: the Matlab function std(.) is not used here for clarity.
dd1 = d1 - dm;
dd2 = d2 - dm;
dd3 = d3 - dm;
for i = 1:M
    sigma(i) = sqrt((dd1(i)^2 + dd2(i)^2 + dd3(i)^2)/(N-1));
    %sigma_tmp(i) = std([dd1(i) ; dd2(i) ; dd3(i)]);    % for verification
end

% Fourth step: a) path (positioning) repeatability rtp(i) at the i-th position
% for N attained paths, and b) the total path rpeatability rT
rtp = dm + 3*sigma;
rt_2D = max(rtp);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                          Display results                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[ymax,iymax] = max(x1(2,:));
[ymin,iymin] = min(x1(2,:));
[zmax,izmax] = max(x1(3,:));
[zmin,izmin] = min(x1(3,:));
disp(' ')
disp('******************************************************************')
disp('***         Copyright Cambridge University Press 2010          ***')
disp('*** Karl Mathia (2010). Robotics for Electronics Manufacturing ***')
disp('***                  Chapter 7, Example 7.7                    ***')
disp('******************************************************************')
disp(' ')
disp('Computing the path positioning accuracy and repeatability of a') 
disp('straight 3D path from real-world 3D test data per ISO 9283.')
disp('Please refer to the description in the book for further details.')
disp(' ')
fprintf('Attained paths x_j, j=1,..,N:      N  = %d\n', N)
fprintf('Samples per path x_j(i), i=1,..,M: M  = %d\n', M)
disp(' ')
fprintf('Path positioning data in 3D (x, y, z):\n')
fprintf('3D results:\n')
fprintf('  Path accuracy:       at = %5.0f [um]\n', at_3D);
fprintf('  Path repeatability:  rt = %5.0f [um]\n', rt_3D);
disp(' ')
fprintf('Path positioning data in 2D (x=0, y, z):\n')
fprintf('2D results:\n')
fprintf('  Path accuracy:        at = %5.0f [um] at x = %5.3f m\n', at_2D, 1e-6*x1(1,imax_2D));
fprintf('  Path repeatability:   rt = %5.0f [um]\n', rt_2D);
fprintf('  Max horizontal error: max(y) = %5.0f [um] at i = %d (%5.3f m)\n', ymax, iymax, 1e-6*x1(1,iymax));
fprintf('  Min horizontal error: min(y) = %5.0f [um] at i = %d (%5.3f m)\n', ymin, iymin, 1e-6*x1(1,iymin));
fprintf('  Max vertical error:   max(z) = %5.0f [um] at i = %d (%5.3f m)\n', zmax, izmax, 1e-6*x1(1,izmax));
fprintf('  Min vertical error:   min(z) = %5.0f [um] at i = %d (%5.3f m)\n', zmin, izmin, 1e-6*x1(1,izmin));
disp('******************************************************************')
disp(' ')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           Plot 2D paths
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Smoothen mean path with moving average for the graph 
a = 1;
b = [1/8 1/8 1/8 1/8 1/8 1/8 1/8 1/8];
xm_ma(1,:) = filter(b,a,xm(1,:));
xm_ma(2,:) = filter(b,a,xm(2,:));
xm_ma(3,:) = filter(b,a,xm(3,:));
xd_ma(1,:) = filter(b,a,xd(1,:));
xd_ma(2,:) = filter(b,a,xd(2,:));
xd_ma(3,:) = filter(b,a,xd(3,:));

% plot x-axis in [meter], but y and z in [micrometer]
axislimits = [1e-6*min(x1(1,:)) 1e-6*max(x1(1,:)) 1.25*min(x1(2,:)) 1.1*max(x1(2,:))];

figure(1)
% plot x-axis in [meter], but y and z in [micrometer]
plot(1e-6*xc(1,:),xc(2,:),'k')
hold on
plot(1e-6*x1(1,:),x1(2,:),'ok','MarkerSize',2)
plot(1e-6*xm_ma(1,:),xm_ma(2,:),'-k','Linewidth',2)
plot(1e-6*[xc(1,imax_3D),x1(1,imax_3D)],[xc(2,imax_3D),x1(2,imax_3D)],'-k')
plot(1e-6*[xc(1,imax_3D),x1(1,imax_3D)],[xc(2,imax_3D),x1(2,imax_3D)],'ok')
plot(1e-6*[xc(1,imax_2D),x1(1,imax_2D)],[xc(2,imax_2D),x1(2,imax_2D)],'-k')
plot(1e-6*[xc(1,imax_2D),x1(1,imax_2D)],[xc(2,imax_2D),x1(2,imax_2D)],'ok')
hold off
axis(axislimits)
xlabel('x [m]')
ylabel('y [um]')

figure(2)
% plot x-axis in [meter], but y and z in [micrometer]
plot(1e-6*xc(1,:),xc(3,:),'k')
hold on
plot(1e-6*x1(1,:),x1(3,:),'ok','MarkerSize',2)
plot(1e-6*xm_ma(1,:),xm_ma(3,:),'-k','Linewidth',2)
plot(1e-6*[xc(1,imax_3D),x1(1,imax_3D)],[xc(3,imax_3D),x1(3,imax_3D)],'-k')
plot(1e-6*[xc(1,imax_3D),x1(1,imax_3D)],[xc(3,imax_3D),x1(3,imax_3D)],'ok')
plot(1e-6*[xc(1,imax_2D),x1(1,imax_2D)],[xc(3,imax_2D),x1(3,imax_2D)],'-k')
plot(1e-6*[xc(1,imax_2D),x1(1,imax_2D)],[xc(3,imax_2D),x1(3,imax_2D)],'ok')
hold off
axis(axislimits)
xlabel('x [m]')
ylabel('z [um]')

% plot dynamic, attained path in horizontal x/y plane against static path
figure(3)
% plot x-axis in [meter], but y and z in [micrometer]
plot(1e-6*xd_ma(1,:),xd_ma(2,:),'k','Linewidth',2)
hold on
plot(1e-6*xm_ma(1,:),xm_ma(2,:),'k')
hold off
axis(axislimits)
xlabel('x [m]')
ylabel('y [um]')

% plot dynamic, attained path in vertical x/z plane against static path
figure(4)
% plot x-axis in [meter], but y and z in [micrometer]
plot(1e-6*xd_ma(1,:),xd_ma(3,:),'k','Linewidth',2)
hold on
plot(1e-6*xm_ma(1,:),xm_ma(3,:),'k')
hold off
axis(axislimits)
xlabel('x [m]')
ylabel('y [um]')

%%%
