%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Cambridge University Press 2010
% Book: Karl Mathia (2010), Robotics for Electronics Manufacturing - 
% Principles and Applications in Cleanroom Manufacturing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Chapter 7, Example 7.8:
% Vibration analysis of a wafer-handling robot arm and end-effector in both
% time domain and frequency domain.
% Refer to the description in the book for further details.
%
% The following code can be executed with Matlab(R)* or GNU Octave. The
% Octave plot function uses gnuplot and may cause the plot window to freeze
% in rare cases.
% *Matlab(R) is a registered trademark of The MathWorks.

clear
clc
    

% Load data file: acceleration in robot's polar coordinate frame (Theta, 
% R, Z)
load acceleration_over_time.dat     % 3 acceleration signals and E.E. position
aT = acceleration_over_time(:,1);   % accel. on T-axis (rotation) in [g]
aR = acceleration_over_time(:,2);   % accel. on R-axis (radial) in [g]
aZ = acceleration_over_time(:,3);   % accel. on Z-axis (vertical, or Z) in [g]
d  = acceleration_over_time(:,4);   % displacement at commanded pose in [#]
clear acceleration_over_time

% determine sampling period and time axis
N = length(aR);             % length of time series (and DFT)
bandwidth = 500;            % Hertz
fs = 2.56 * bandwidth;      % sampling frequency: 2.56 * 500Hz = 1280 Hz
T = 1/fs;                   % sampling period in [sec]
time = T*[0:N-1];           % time axis in [sec]

%%% Time domain calculations %%%

% determine time period of controlled robot motion plus settling period
Tsettle = 0.5;              % selected time period for settling [s]
t0 = 0.20;                  % start time for display x-axis [sec]
t1 = 0.24;                  % motion start time [sec]
t2 = 0.82;                  % motion end time [sec]
t3 = t2 + Tsettle;          % end of settling time window [sec]
tmid = (t1+t2)/2;           % mid-point of controlled motion trajectory
N0 = round(t0/T);           % start sample for display
N1 = round(t1/T);           % motion start sample
N2 = round(t2/T);           % motion end sample
N3 = round(t3/T);           % settling end sample

% convert acceleration from [g] to [m/s^2]
aT = 9.81 * aT;             % T-axis (rotation) in [m/s^2]
aR = 9.81 * aR;             % R-axis (radial) in [m/s^2]
aZ = 9.81 * aZ;             % Z-axis (vertical, or Z) in [m/s^2]

% 2-point moving average (MA): filter time series to reduce noise
aa = 1;
bb = [1/2 1/2];
aRma = filter(bb,aa,aR);
aTma = filter(bb,aa,aT);
aZma = filter(bb,aa,aZ);



%%% Frequency domain calculations %%%


% Fast Fourier Transform of finite-length time series and its movinng
% average (MA)
aT_fft = fft(aT);
aR_fft = fft(aR);
aZ_fft = fft(aZ);
aTma_fft = fft(aTma);
aRma_fft = fft(aRma);
aZma_fft = fft(aZma);
aTma_fft_abs = 1/N * abs(aTma_fft);
aRma_fft_abs = 1/N * abs(aRma_fft);
aZma_fft_abs = 1/N * abs(aZma_fft);

% Power spectrum in decibel (dB)
aTma_Pyy = 1/N * aTma_fft .* conj(aTma_fft);
aRma_Pyy = 1/N * aRma_fft .* conj(aRma_fft);
aZma_Pyy = 1/N * aZma_fft .* conj(aZma_fft);

% frequency axis for display
freq = fs/N * [0:N/2];

%%% Performance indices %%%

% maximum absolute values 
aTmamax = max(abs(aTma));
aRmamax = max(abs(aRma));
aZmamax = max(abs(aZma));


% Determine boundaries for time and frequency domains time series
ymin_t = min([aTma;aRma;aZma]);
if ymin_t > 0
    ymin_t = 0.95 * ymin_t;
else
    ymin_t = 1.05 * ymin_t;
end

ymax_t = max([aTma;aRma;aZma]);
if ymax_t > 0
    ymax_t = 1.05 * ymax_t;
else
    ymax_t = 0.95 * ymax_t;
end

% Discrete Fourier Transform
ymin_f = min([aRma_fft_abs;aRma_fft_abs;aZma_fft_abs]);
if ymin_f > 0
    ymin_f = 0.95 * ymin_f;
else
    ymin_f = 1.05 * ymin_f;
end

ymax_f = 1.05 * max([aRma_fft_abs;aRma_fft_abs;aZma_fft_abs]);
if ymax_f > 0
    ymax_f = 1.05 * ymax_f;
else
    ymax_f = 0.95 * ymax_f;
end


%%% Display results %%%

disp('******************************************************************')
disp('***         Copyright Cambridge University Press 2010          ***')
disp('*** Karl Mathia (2010). Robotics for Electronics Manufacturing ***')
disp('***                  Chapter 7, Example 7.8                    ***')
disp('******************************************************************')
disp(' ')
disp('Vibration analysis of a wafer-handling robot arm and end-effector')
disp('in both time domain and frequency domain.')
disp('Please refer to the description in the book for further details.')
disp(' ')
disp('Plotting...')
disp(' ')
disp('******************************************************************')
disp(' ')

figure(1)
% y-axis (T-axis): plot acceleration time series (moving average)
subplot(2,1,1)
hold on
%plot(time(N0:N3),aTma(N0:N3,1),'k','Linewidth',2)
plot(time(N0:N3),aTma(N0:N3,1),'k')
plot([t1,t1],[ymin_t,ymax_t],'k--')
plot([t2,t2],[ymin_t,ymax_t],'k--')
hold off
axis([t0 t3 ymin_t ymax_t])
grid on
xlabel('time [s]')
ylabel('T-accel. [m/s^2]')

% x-axis (R-axis): plot acceleration time series
subplot(2,1,2)
hold on
%plot(time(N0:N3),aRma(N0:N3,1),'k','Linewidth',2)
plot(time(N0:N3),aRma(N0:N3,1),'k')
plot([t1,t1],[ymin_t,ymax_t],'k--')
plot([t2,t2],[ymin_t,ymax_t],'k--')
plot(tmid,0,'ok')
hold off
axis([t0 t3 ymin_t ymax_t])
grid on
xlabel('time [s]')
ylabel('R-accel. [m/s^2]')

figure(2)
% z-axis: plot acceleration time series
subplot(2,1,1)
hold on
%plot(time(N0:N3),aZma(N0:N3,1),'k','Linewidth',2)
plot(time(N0:N3),aZma(N0:N3,1),'k')
plot([t1,t1],[ymin_t ymax_t],'k--')
plot([t2,t2],[ymin_t ymax_t],'k--')
hold off
axis([t0 t3 ymin_t ymax_t])
grid on
xlabel('time [s]')
ylabel('Z-accel. [m/s^2]')

% vertical end-effector displacement
subplot(2,1,2)
%plot(time(N0:N3),d(N0:N3,1),'k','Linewidth',2)
plot(time(N0:N3),d(N0:N3,1),'k')
hold on
plot([t1,t1],[ymin_t,ymax_t],'k--')
plot([t2,t2],[ymin_t,ymax_t],'k--')
hold off
axis([t0 t3 ymin_t ymax_t])
grid on
xlabel('time [s]')
ylabel('Settling: d(t) [um]')


% display Discrete Fourier Transform (DFT)
figure(3)
subplot(2,1,1)
%plot(freq,aTma_fft_abs(1:N/2+1),'k','Linewidth',2)
plot(freq,aTma_fft_abs(1:N/2+1),'k')
axis([0 100 ymin_f ymax_f])
grid on
xlabel('frequency [Hz]')
ylabel('T-amplitude [m/s^2]')
subplot(2,1,2)
%plot(freq,aRma_fft_abs(1:N/2+1),'k','Linewidth',2)
plot(freq,aRma_fft_abs(1:N/2+1),'k')
axis([0 100 ymin_f ymax_f])
grid on
xlabel('frequency [Hz]')
ylabel('R-amplitude [m/s^2]')

figure(4)
subplot(2,1,1)
%plot(freq,aZma_fft_abs(1:N/2+1),'k','Linewidth',2)
plot(freq,aZma_fft_abs(1:N/2+1),'k')
axis([0 100 ymin_f ymax_f])
grid on
xlabel('frequency [Hz]')
ylabel('Z-amplitude [m/s^2]')

%%%